## This script builds material graph from .pickle and save it in .dot format.
## Example cmd: python.exe script_build_material_graph.py "path_to_material\EDM_Default_Material.pikle".
## You will get "path_to_material\EDM_Default_Material.dot".
## To see the results insert data from .dot in https://edotor.net/.

import re
import os
import sys
from typing import Dict, Set, List, Tuple
from pathlib import Path

import pickle
import graphviz 

sys.path.append(str(Path(__file__).absolute().parent)+"\\..")
pattern_format_node = re.compile(r'[^\w"]')

def add_node(nodes: Dict, node: str, socket: str):
    full_s_name = pattern_format_node.sub('_', f'{node}_{socket}')

    s = (full_s_name, socket)
    if node in nodes:
        nodes[node].add(s)
    else:
        nodes[node] = {s}

    return f'{node}:{full_s_name}'


def from_dict_to_str(node: str, nodes: Dict):
    if node not in nodes:
        return ''
    return '{' + '|'.join([f'<{full_s}> {s}' for (full_s, s) in nodes[node]]) + '}'


## check argument
args = sys.argv
input_pickle_path = args[1]
if os.path.splitext(input_pickle_path)[1] != ".pickle":
    print("\nError: Wrong arguments. Try again.\n It must be \"script_build_material_graph.py mat_desc_file.pikle\"")
    exit()

#input_pickle_path = rf'e:\repos\trunk\Utils\EDMTools\io_scene_edm\data\EDM_Default_Material.pickle'
dir_path, file_name = os.path.dirname(input_pickle_path), os.path.splitext(os.path.basename(input_pickle_path))[0]
output_dot_file = os.path.join(dir_path, file_name + '.dot')

## run parsing
with open(input_pickle_path, 'rb') as f:
    mat_desc = pickle.load(f)

dot = graphviz.Digraph(file_name, node_attr={'shape': 'record'})

nodes_in: Dict[str, Set[Tuple[str, str]]] = {}
nodes_out: Dict[str, Set[Tuple[str, str]]] = {}
links: List[Tuple[str, str]] = []
node_names: List[str] = []

for link in mat_desc.links:
    full_from = add_node(nodes_out, link.from_node, link.from_socket)
    full_to = add_node(nodes_in, link.to_node, link.to_socket)
    links.append((full_from, full_to))
    node_names.append(link.from_node)
    node_names.append(link.to_node)

for node in node_names:
    sockets_in = from_dict_to_str(node, nodes_in)    
    sockets_out = from_dict_to_str(node, nodes_out)

    sockets = rf"{{{node}|{{{sockets_in}|{sockets_out}}}}}"
    dot.node(node, sockets)

dot.edges(links)

try:
    dot.render(filename=output_dot_file)
except:
    pass
